require_relative 'lib/docker'

include Docker::Helpers

def get_args(num_args)
  # See this link for explanation of this black magic:
  # http://itshouldbeuseful.wordpress.com/2011/11/07/passing-parameters-to-a-rake-task/
  args = ARGV.last(num_args)
  yield(args)
  args.each { |arg| task arg.to_sym {} }
end

def build_docker_app_image(type, app_path)
  full_path_and_name(app_path) do |path, name|
    `docker build -t jphager2/#{type}-#{name} #{path}`
  end
end

# You Can't overwrite rake tasks (yet)
# Paths Can't be relative (try to source them)
namespace :docker do
  desc "Stop all running docker containers and then remove them"
  task :cleanup do
    Docker.containers.each { |c| c.remove! }
  end

  desc "Remove all nameless docker images (repositories, tags)"
  task :nameless do
    Docker.images(:repository).select {|r| r.id == "<none>"}.each do |r|
      r.remove_image!
    end
  end
end
