require_relative 'docker/rake'
require_relative 'docker/helpers'
require_relative 'docker/interactive'
require_relative 'docker/equality'
require_relative 'docker/path'
require_relative 'docker/id'

require_relative 'docker/image'
require_relative 'docker/repository'
require_relative 'docker/tag'
require_relative 'docker/container'
require_relative 'docker/inspect'

require_relative 'docker/rails'
require_relative 'docker/rails/server'
require_relative 'docker/rails/console'
require_relative 'docker/ruby'
require_relative 'docker/postgres'

module Docker

  extend Helpers
  extend self 

  USER_NAME    = 'jphager2'

  def ID(input)
    case input
    when String
      Id.create(input)
    when ->(obj){obj.respond_to?(:id)}
      Id.create(input.id)
    end
  end

  def PATH(input)
    case input
    when String
      Path.new(input)
    when Path
      input
    end
  end

  # delegate to Image
  def build(app_path, image_tag = nil)
    path = PATH(app_path)
    image_tag ||= path.name
    Image.build(path, image_tag)
  end

  # delegate to Container
  def run(image, options = "", command = "")
    Container.run(image.to_s, options, command)
  end

  def images(response_type = :image)
    images = `docker images`.split("\n")[1..-1]
    images.map do |image|
      id    = image.match(/^[^\s]+\s+[^\s]+\s+([^\s]+)/)[1]
      repo  = image.match(/^[a-zA-Z0-9\/\-_\.<>]+/)[0]
      tag   = image.match(/^[^\s]+\s+([^\s]+)/)[1]
      case response_type
      when :repository
        Repository.new(id, repo)
      when :tag
        Tag.new(id, repo, tag)
      when :image
        Image.new(id)
      end
    end
  end

  def inspekt(id)
    inspekt = Inspect.new(id.to_s)
    inspekt.inspekt ? inspekt : NullInspect.new
  end

  def validate_search_parameters(options)
    bad_args = options.keys.select {|key| key.to_s.end_with?('!')}
    raise ArgumentError, 'bang method given ' +
      "as option: #{bad_args.join(', ')}" unless bad_args.empty?
  end

  def find_image(options = {})
    klass = options.delete(:klass) || :image
    validate_search_parameters(options)
    images(klass).find do |image|
      found = true 
      options.each do |option,value| 
        found = false unless image.send(option) == value 
      end
      found
    end
  end

  def find_container(options = {})
    validate_search_parameters(options)
    containers.find do |container|
      found = true 
      options.each do |option,value| 
        found = false unless container.send(option) == value 
      end
      found
    end
  end

  def find_containers(options = {})
    klass = options.delete(:klass)
    validate_search_parameters(options)
    kontainers = containers
    kontainers.map! { |container| klass.load_from(container) } if klass
    kontainers.select { |container|
      found = true 
      options.each do |option, value|
        found = false unless container.send(option) === value
      end
      found
    }.compact 
  end

  def containers(mode = :all)
    containers = case mode
    when :all
      all_containers
    when :stopped
      all_containers.select { |container| container =~ /Exited/ }
    when :paused
      all_containers.select { |container| container =~ /(Paused)/ }
    when :running
      `docker ps`.split("\n")[1..-1]
        .select { |container| container !~ /(Paused)/}
    else
      raise ArgumentError, "Bad Argument: #{mode}" 
    end
    containers.map do |container| 
      Container.new(container[0..11]).to_klass 
    end 
  end

  def all_containers
    `docker ps -a`.split("\n")[1..-1]
  end
end
