module Docker
  class Container
    include Docker::Equality

    attr_reader :id, :removal_instructions
    def initialize(id)
      @id = Docker::ID(id) 
    end

    def self.run(image, flags = "", command = "", path = nil)
      path  = Docker::PATH(path)
      flags = "#{flags} -e DOCKER_TYPE=#{self}"
      flags = "#{flags} -e LOCAL_PATH=#{path.path}" if path

      id = Docker::ID(`docker run #{flags} #{image} #{command}`)

      # Nasty hack to return a removed container if the --rm flag
      # is passed and the container alread exited
      return new(nil).disable! unless id || flags !~ /--rm/

      id ||= Docker.containers(:stopped).first.id

      new(id) 
    end

    def self.load_from(container)
      new(container.id) if container.from_a?(self)
    end

    def logs
      @logs || `docker logs #{id}` # @logs given with #disable!
    end

    def inspekt
      Docker.inspekt(id)
    end

    def to_klass
      return self unless klass = from_klass
      klass.load_from(self)
    end

    def from_klass
      return nil unless klass = env["DOCKER_TYPE"]
      Kernel.const_get(klass)
    end

    def path
      Docker::PATH(env["LOCAL_PATH"])
    end

    def add_removal_instructions(&block)
      @removal_instructions = Proc.new { block.call(self) }
      self
    end

    def attach!
      `docker attach --sig-proxy=false #{id}`
    end

    def remove!
      @logs = logs
      @removal_instructions.call if @removal_instructions
      `docker rm -f #{id}`
      disable!
      id
    end

    def disable!
      self.instance_eval { 
        undef :attach!
        undef :remove!
        undef :add_removal_instructions
      }
      self.instance_eval { def state; @id; end } 
      @id   = 'REMOVED'
      self
    end

    def from_a?(klass)
      env["DOCKER_TYPE"] == klass.to_s
    end

    def removed?
      id == "REMOVED"
    end

    def exists_in?(state)
      Docker.containers(state).any?  { |container| container == self }
    end

    def up?; running?; end
    def ip; ip_address; end
    alias_method :to_s, :id

    def method_missing(method, *args, &block)
      super if removed?
      inspekt_methods = [:env, :state, :ports, :ip_address, :name]
      state_methods   = [:running? , :stopped?, :paused?]
      action_methods  = [:stop!, :start!, :restart!, :pause!, :unpause!]
      if inspekt_methods.include?(method)
        inspekt.__send__(method) 
      elsif state_methods.include?(method)
        method = method.to_s.delete('?').to_sym
        exists_in?(method) 
      elsif action_methods.include?(method)
        method = method.to_s.delete('!').to_sym
        `docker #{method} #{id}`
      else
        super
      end
    end
  end
end
