module Docker
  module Rails
    class Server < Docker::Container

      extend Docker::Helpers
      
      def initialize(id)
        super
      end

      def self.run(path, postgres, options = "")
        server = existing_server(path) 
        return server if server
        migrate_database!(path, postgres)
        server = super(
          "#{rails_image(path.name)}", 
          options                                               +
          " -d --name #{rails_server(path.name)} --link db:db " +
          "-e PG_HOST=#{postgres.ip} -v #{path}:/usr/src/app",
          nil,
          path
        )
        self.load(server.id, path)
      end

      def self.load_from(container)
        load(container.id, container.path)
      end

      def self.load(id, path)
        server = new(id)
        Docker::Rails.add_default_instructions_to(server, path)
      end

      def self.migrate_database!(path, postgres)
        ['create', 'migrate'].each do |cmd|
          Docker.run(
            "#{rails_image(path.name)}", 
            "--rm --link db:db -e PG_HOST=#{postgres.ip} " + 
            "-v #{path}:/usr/src/app",
            "rake db:#{cmd}"
          )
        end
      end

      def self.existing_server(path)
        server = Rails.find_server(path)
        return server if server && server.up?
        server.remove! if server
        nil
      end
    end
  end
end
